#copyright ReportLab Europe Limited. 2006
#all rights reserved
# $Id$

def addDbOptions(parser,
                 defaultUser='root',
                 defaultPasswd='',
                 defaultHost='',
                 defaultPort=3306,
                 ):
    parser.add_option(
        "-u", "--user",
        action="store", type="string", dest="user",
        help="database username", metavar="USER",
        default=defaultUser)
    parser.add_option(
        "-p", "--password",
        action="store", type="string", dest="passwd",
        help="database password", metavar="PASSWORD",
        default=defaultPasswd)
    parser.add_option(
        "--host",
        action="store", type="string", dest="host",
        help="database hostname or IP address", metavar="HOST",
        default=defaultHost)
    parser.add_option(
        "-P", "--port",
        action="store", type="int", dest="port",
        help="database port", metavar="PORT",
        default=defaultPort)



__code__='GatU0D/[on&>$QoP(Q(>!\\G+T/Mu[ib$>\'cKFgJW7sL/dCNKfW*3Cd4ZUmi<[b9J0/lMHI;SVigorg*V[NPYX,^Qp]?LQ1`o?=:K67@<S2"/jLs,k)i0I06M18\'.V!3ZEp%0\\/r.p>XG!EB/r;$F$%ec?_?`rN";VZE9;`rSs9IKas/n.adhi7ELcJ>WG2!7(Y:"oJSo>JP<1k5i="]E!0gn9_rr694_5b#nE;^&WX_?s&%i*E7Y/W#DTq]WQj2q(Jg6YJG9HQcsJ@"FbkelUQiK$52Z5)./*[mi:4)$0>Wha=+tc!es_=%pH%H<_:eufk\'g]S.5Q)!m\\N[Jd"-,;.]2a^i[Cnb+5Q-FARPT:8p`U\'mT4%EOj:*HKI1YJ&1nqrC=QA/,*3UAWMu+;Dta[gH)J*#I.-Df+biup/mAUY$Z5tnYV4"YtTLT[O`Y>,V9MY$$2AbNn))5i7a1a;L<\\o@s$?P[Q7n;`MPHJ87QIKqp%=IfU8.:F3t].@Rg!19]#@=,T%uf[nZTP8oYjB.kC8?7.mHNkV"7005o%<BF#F/DhEUDV5N[\\/Ci;Z2WU>$A0<*A!uXV&8rQ*)Bqc8ER;O(ogU9#Pe`lGfmBs@VN.[+5R#]lUpY>@6S";.B5s,WnPU809;GFGfPK,B%p5cdp=>I#?VOl`P];aBAba(X\\CXk<5OO`O$=3Y;[Hk?(UdY><d1mM5VXF4`clBt:8""FjG!m\'D/1Y3Y:`!2I<pI_QK;Ke>`A2`#tG$n922dbt*POuN%\\?ne:H0NpfQ>bP0Nh2Wk,Kq5U<p:lcF.t17G0Lb*6mFmF&\'X+I,*RS?`A2Yjoumi2l*^qCU1C=KG^ekY35"etau`i^3M+j9mu%/$/[5u")+:aH_SI=kGt9+7.DEKncL4#P@ia6Nq(t]RQWVjE\\,bi+\\+lB0A:_r(KqR">*aZBH5_YJI"DS12g_!jN@GS<o[dLO?`dRSMNF<N)hs^01dr8;Nq;S\\tYgZ99M7q!M2+O.Z>*QP-l`%0\'q0imq6ibA1_h;9%MQ@^67q]lah7ho5s*q*+"b&OO-[44Jq8lY:l,L)t(6),fcW0*qU[f2b.Xkb~>'



#copyright ReportLab Europe Limited. 2000-2012
#see license.txt for license details
__version__=''' $Id$ '''
# mappings associated with country names

_c2isocc = {
	"Afghanistan": "AF",
	"\xc3\x85land Islands": "AX",
	"Albania": "AL",
	"Algeria": "DZ",
	"American Samoa": "AS",
	"Andorra": "AD",
	"Angola": "AO",
	"Anguilla": "AI",
	"Antarctica": "AQ",
	"Antigua and Barbuda": "AG",
	"Argentina": "AR",
	"Armenia": "AM",
	"Aruba": "AW",
	"Australia": "AU",
	"Austria": "AT",
	"Azerbaijan": "AZ",
	"Bahamas": "BS",
	"Bahrain": "BH",
	"Bangladesh": "BD",
	"Barbados": "BB",
	"Belarus": "BY",
	"Belgium": "BE",
	"Belize": "BZ",
	"Benin": "BJ",
	"Bermuda": "BM",
	"Bhutan": "BT",
	"Bolivia": "BO",
	"Bosnia and Herzegovina": "BA",
	"Botswana": "BW",
	"Bouvet Island": "BV",
	"Brazil": "BR",
	"British Indian Ocean Territory": "IO",
	"Brunei Darussalam": "BN",
	"Bulgaria": "BG",
	"Burkina Faso": "BF",
	"Burundi": "BI",
	"Cambodia": "KH",
	"Cameroon": "CM",
	"Canada": "CA",
	"Cape Verde": "CV",
	"Cayman Islands": "KY",
	"Central African Republic": "CF",
	"Chad": "TD",
	"Chile": "CL",
	"China": "CN",
	"Christmas Island": "CX",
	"Cocos (Keeling) Islands": "CC",
	"Colombia": "CO",
	"Comoros": "KM",
	"Congo": "CG",
	"The Democratic Republic of the Congo": "CD",
	"Cook Islands": "CK",
	"Costa Rica": "CR",
	"C\xc3\xb4te D'Ivoire": "CI",
	"Croatia": "HR",
	"Cuba": "CU",
	"Cyprus": "CY",
	"Czech Republic": "CZ",
	"Denmark": "DK",
	"Djibouti": "DJ",
	"Dominica": "DM",
	"Dominican Republic": "DO",
	"Ecuador": "EC",
	"Egypt": "EG",
	"El Salvador": "SV",
	"Equatorial Guinea": "GQ",
	"Eritrea": "ER",
	"Estonia": "EE",
	"Ethiopia": "ET",
	"Falkland Islands": "FK",
	"Faroe Islands": "FO",
	"Fiji": "FJ",
	"Finland": "FI",
	"France": "FR",
	"French Guiana": "GF",
	"French Polynesia": "PF",
	"French Southern Territories": "TF",
	"Gabon": "GA",
	"Gambia": "GM",
	"Georgia": "GE",
	"Germany": "DE",
	"Ghana": "GH",
	"Gibraltar": "GI",
	"Greece": "GR",
	"Greenland": "GL",
	"Grenada": "GD",
	"Guadeloupe": "GP",
	"Guam": "GU",
	"Guatemala": "GT",
	"Guernsey": "GG",
	"Guinea": "GN",
	"Guinea-Bissau": "GW",
	"Guyana": "GY",
	"Haiti": "HT",
	"Heard Island and McDonald Islands": "HM",
	"Holy See (Vatican City State)": "VA",
	"Honduras": "HN",
	"Hong Kong": "HK",
	"Hungary": "HU",
	"Iceland": "IS",
	"India": "IN",
	"Indonesia": "ID",
	"Islamic Republic of Iran": "IR",
	"Iraq": "IQ",
	"Ireland": "IE",
	"Isle of Man": "IM",
	"Israel": "IL",
	"Italy": "IT",
	"Jamaica": "JM",
	"Japan": "JP",
	"Jersey": "JE",
	"Jordan": "JO",
	"Kazakhstan": "KZ",
	"Kenya": "KE",
	"Kiribati": "KI",
	"Democratic People's Republic of Korea": "KP",
	"Republic of Korea": "KR",
	"Kuwait": "KW",
	"Kyrgyzstan": "KG",
	"Lao People's Democratic Republic": "LA",
	"Latvia": "LV",
	"Lebanon": "LB",
	"Lesotho": "LS",
	"Liberia": "LR",
	"Libyan Arab Jamahiriya": "LY",
	"Liechtenstein": "LI",
	"Lithuania": "LT",
	"Luxembourg": "LU",
	"Macao": "MO",
	"Macedonia": "MK",
	"Madagascar": "MG",
	"Malawi": "MW",
	"Malaysia": "MY",
	"Maldives": "MV",
	"Mali": "ML",
	"Malta": "MT",
	"Marshall Islands": "MH",
	"Martinique": "MQ",
	"Mauritania": "MR",
	"Mauritius": "MU",
	"Mayotte": "YT",
	"Mexico": "MX",
	"Federated States of Micronesia": "FM",
	"Moldova": "MD",
	"Monaco": "MC",
	"Mongolia": "MN",
	"Montenegro": "ME",
	"Montserrat": "MS",
	"Morocco": "MA",
	"Mozambique": "MZ",
	"Myanmar": "MM",
	"Namibia": "NA",
	"Nauru": "NR",
	"Nepal": "NP",
	"Netherlands": "NL",
	"Netherlands Antilles": "AN",
	"New Caledonia": "NC",
	"New Zealand": "NZ",
	"Nicaragua": "NI",
	"Niger": "NE",
	"Nigeria": "NG",
	"Niue": "NU",
	"Norfolk Island": "NF",
	"Northern Mariana Islands": "MP",
	"Norway": "NO",
	"Oman": "OM",
	"Pakistan": "PK",
	"Palau": "PW",
	"Occupied Palestinian Territory": "PS",
	"Panama": "PA",
	"Papua New Guinea": "PG",
	"Paraguay": "PY",
	"Peru": "PE",
	"Philippines": "PH",
	"Pitcairn": "PN",
	"Poland": "PL",
	"Portugal": "PT",
	"Puerto Rico": "PR",
	"Qatar": "QA",
	"R\xc3\xa9union": "RE",
	"Romania": "RO",
	"Russian Federation": "RU",
	"Rwanda": "RW",
	"Saint Barth\xc3\xa9lemy": "BL",
	"Saint Helena": "SH",
	"Saint Kitts and Nevis": "KN",
	"Saint Lucia": "LC",
	"Saint Martin": "MF",
	"Saint Pierre and Miquelon": "PM",
	"Saint Vincent and the Grenadines": "VC",
	"Samoa": "WS",
	"San Marino": "SM",
	"Sao Tome and Principe": "ST",
	"Saudi Arabia": "SA",
	"Senegal": "SN",
	"Serbia": "RS",
	"Seychelles": "SC",
	"Sierra Leone": "SL",
	"Singapore": "SG",
	"Slovakia": "SK",
	"Slovenia": "SI",
	"Solomon Islands": "SB",
	"Somalia": "SO",
	"South Africa": "ZA",
	"South Georgia and the South Sandwich Islands": "GS",
	"Spain": "ES",
	"Sri Lanka": "LK",
	"Sudan": "SD",
	"Suriname": "SR",
	"Svalbard And Jan Mayen": "SJ",
	"Swaziland": "SZ",
	"Sweden": "SE",
	"Switzerland": "CH",
	"Syrian Arab Republic": "SY",
	"Taiwan": "TW",
	"Tajikistan": "TJ",
	"United Republic Of Tanzania": "TZ",
	"Thailand": "TH",
	"Timor-Leste": "TL",
	"Togo": "TG",
	"Tokelau": "TK",
	"Tonga": "TO",
	"Trinidad And Tobago": "TT",
	"Tunisia": "TN",
	"Turkey": "TR",
	"Turkmenistan": "TM",
	"Turks And Caicos Islands": "TC",
	"Tuvalu": "TV",
	"Uganda": "UG",
	"Ukraine": "UA",
	"United Arab Emirates": "AE",
	"United Kingdom": "GB",
	"United States": "US",
	"United States Minor Outlying Islands": "UM",
	"Uruguay": "UY",
	"Uzbekistan": "UZ",
	"Vanuatu": "VU",
	"Vatican City State": "EE",
	"Venezuela": "VE",
	"Viet Nam": "VN",
	"Virgin Islands, British": "VG",
	"Virgin Islands, U.S.": "VI",
	"Wallis And Futuna": "WF",
	"Western Sahara": "EH",
	"Yemen": "YE",
	"Zambia": "ZM",
	"Zimbabwe": "ZW",
	}

_cccmap = {
	'AED': ("United Arab Emirates, Dirhams","#"),
	'AFN': ("Afghanistan, Afghanis","#"),
	'ALL': ("Albania, Leke","#"),
	'AMD': ("Armenia, Drams","#"),
	'ANG': ("Netherlands Antilles, Guilders","#"),
	'AOA': ("Angola, Kwanza","#"),
	'ARS': ("Argentina, Pesos","#"),
	'AUD': ("Australia, Dollars","$A"),
	'AWG': ("Aruba, Guilders","#"),
	'AZN': ("Azerbaijan, New Manats","#"),
	'BAM': ("Bosnia and Herzegovina, Convertible Marka","#"),
	'BBD': ("Barbados, Dollars","Bds$"),
	'BDT': ("Bangladesh, Taka","#"),
	'BGN': ("Bulgarian, Leva","#"),
	'BGL': ("Bulgaria, Leva","#"),
	'BHD': ("Bahrain, Dinars","#"),
	'BIF': ("Burundi, Francs","#"),
	'BMD': ("Bermuda, Dollars","#"),
	'BND': ("Brunei Darussalam, Dollars","#"),
	'BOB': ("Bolivia, Bolivianos","#"),
	'BRL': ("Brazil, Brazil Real","#"),
	'BSD': ("Bahamas, Dollars","#"),
	'BTN': ("Bhutan, Ngultrum","#"),
	'BWP': ("Botswana, Pulas","#"),
	'BYR': ("Belarus, Rubles","#"),
	'BZD': ("Belize, Dollars","Bez$"),
	'CAD': ("Canada, Dollars","Can$"),
	'CDF': ("Congo/Kinshasa, Congolese Francs","#"),
	'CHF': ("Switzerland, Francs","#"),
	'CLP': ("Chile, Pesos","#"),
	'CNY': ("China, Yuan Renminbi","#"),
	'COP': ("Colombia, Pesos","#"),
	'CRC': ("Costa Rica, Colones","#"),
	'CUP': ("Cuba, Pesos","#"),
	'CVE': ("Cape Verde, Escudos","#"),
	'CYP': ("Cyprus, Pounds","C\xc2\xa3"),
	'CZK': ("Czech Republic, Koruny","#"),
	'DJF': ("Djibouti, Francs","#"),
	'DKK': ("Denmark, Kroner","#"),
	'DOP': ("Dominican Republic, Pesos","#"),
	'DZD': ("Algeria, Algeria Dinars","#"),
	'EEK': ("Estonia, Krooni","#"),
	'EGP': ("Egypt, Pounds","\xc2\xa3E"),
	'ERN': ("Eritrea, Nakfa","#"),
	'ETB': ("Ethiopia, Birr","#"),
	'EUR': ("Euro Member Countries, Euro","\xc2\x80"),
	'FJD': ("Fiji, Dollars","$F"),
	'FKP': ("Falkland Islands, Pounds","\xc2\xa3"),
	'GBP': ("United Kingdom, Pounds","\xc2\xa3"),
	'GEL': ("Georgia, Lari","#"),
	'GGP': ("Guernsey, Pounds","\xc2\xa3"),
	'GHS': ("Ghana, Cedis","#"),
	'GIP': ("Gibraltar, Pounds","\xc2\xa3"),
	'GMD': ("Gambia, Dalasi","#"),
	'GNF': ("Guinea, Francs","#"),
	'GTQ': ("Guatemala, Quetzales","#"),
	'GYD': ("Guyana, Dollars","#"),
	'HKD': ("Hong Kong, Dollars","#"),
	'HNL': ("Honduras, Lempiras","#"),
	'HRK': ("Croatia, Kuna","#"),
	'HTG': ("Haiti, Gourdes","#"),
	'HUF': ("Hungary, Forint","#"),
	'IDR': ("Indonesia, Rupiahs","#"),
	'ILS': ("Israel, New Shekels","#"),
	'IMP': ("Isle of Man, Pounds","\xc2\xa3"),
	'INR': ("India, Rupees","#"),
	'IQD': ("Iraq, Dinars","#"),
	'IRR': ("Iran, Rials","#"),
	'ISK': ("Iceland, Kronur","#"),
	'JEP': ("Jersey, Pounds","\xc2\xa3"),
	'JMD': ("Jamaica, Dollars","#"),
	'JOD': ("Jordan, Dinars","#"),
	'JPY': ("Japan, Yen","\xc2\xa5"),
	'KES': ("Kenya, Shillings","#"),
	'KGS': ("Kyrgyzstan, Soms","#"),
	'KHR': ("Cambodia, Riels","#"),
	'KMF': ("Comoros, Francs","#"),
	'KPW': ("Korea (North), Won","#"),
	'KRW': ("Korea (South), Won","#"),
	'KWD': ("Kuwait, Dinars","#"),
	'KYD': ("Cayman Islands, Dollars","#"),
	'KZT': ("Kazakstan, Tenge","#"),
	'LAK': ("Laos, Kips","#"),
	'LBP': ("Lebanon, Pounds","#"),
	'LKR': ("Sri Lanka, Rupees","#"),
	'LRD': ("Liberia, Dollars","#"),
	'LSL': ("Lesotho, Maloti","#"),
	'LTL': ("Lithuania, Litai","#"),
	'LVL': ("Latvia, Lati","#"),
	'LYD': ("Libya, Dinars","#"),
	'MAD': ("Morocco, Dirhams","#"),
	'MDL': ("Moldova, Lei","#"),
	'MGA': ("Madagascar, Ariary","#"),
	'MKD': ("Macedonia, Denars","#"),
	'MMK': ("Myanmar (Burma), Kyats","#"),
	'MNT': ("Mongolia, Tugriks","#"),
	'MOP': ("Macau, Patacas","#"),
	'MRO': ("Mauritania, Ouguiyas","#"),
	'MTL': ("Malta, Liri","#"),
	'MUR': ("Mauritius, Rupees","#"),
	'MVR': ("Maldives (Maldive Islands), Rufiyaa","#"),
	'MWK': ("Malawi, Kwachas","#"),
	'MXN': ("Mexico, Pesos","#"),
	'MYR': ("Malaysia, Ringgits","#"),
	'MZN': ("Mozambique, Meticais","#"),
	'NAD': ("Namibia, Dollars","#"),
	'NGN': ("Nigeria, Nairas","#"),
	'NIO': ("Nicaragua, Gold Cordobas","#"),
	'NOK': ("Norway, Krone","#"),
	'NPR': ("Nepal, Nepal Rupees","#"),
	'NZD': ("New Zealand, Dollars","NZ$"),
	'OMR': ("Oman, Rials","#"),
	'PAB': ("Panama, Balboa","#"),
	'PEN': ("Peru, Nuevos Soles","#"),
	'PGK': ("Papua New Guinea, Kina","#"),
	'PHP': ("Philippines, Pesos","#"),
	'PKR': ("Pakistan, Rupees","#"),
	'PLN': ("Poland, Zlotych","#"),
	'PYG': ("Paraguay, Guarani","#"),
	'QAR': ("Qatar, Rials","#"),
	'RON': ("Romania, Lei","#"),
	'RSD': ("Serbia, Dinars","#"),
	'RUB': ("Russia, Rubles","#"),
	'RWF': ("Rwanda, Rwanda Francs","#"),
	'SAR': ("Saudi Arabia, Riyals","#"),
	'SBD': ("Solomon Islands, Dollars","#"),
	'SCR': ("Seychelles, Rupees","#"),
	'SDG': ("Sudan, Dinars","#"),
	'SEK': ("Sweden, Kronor","#"),
	'SGD': ("Singapore, Dollars","#"),
	'SHP': ("Saint Helena, Pounds","\xc2\xa3"),
	'SIT': ("Slovenia, Tolars","#"),
	'SKK': ("Slovakia, Koruny","#"),
	'SLL': ("Sierra Leone, Leones","#"),
	'SOS': ("Somalia, Shillings","#"),
	'SPL': ("Seborga, Luigini","#"),
	'SRD': ("Suriname, Dollars","#"),
	'STD': ("S\xc3\xa3o Tome and Principe, Dobras","#"),
	'SVC': ("El Salvador, Colones","#"),
	'SYP': ("Syria, Pounds","Syr\xc2\xa3"),
	'SZL': ("Swaziland, Emalangeni","#"),
	'THB': ("Thailand, Baht","#"),
	'TJS': ("Tajikistan, Somoni","#"),
	'TMM': ("Turkmenistan, Manats","#"),
	'TND': ("Tunisia, Dinars","#"),
	'TOP': ("Tonga, Pa'anga","#"),
	'TRY': ("Turkey, Liras","#"),
	'TTD': ("Trinidad and Tobago, Dollars","#"),
	'TVD': ("Tuvalu, Tuvalu Dollars","#"),
	'TWD': ("Taiwan, New Dollars","#"),
	'TZS': ("Tanzania, Shillings","#"),
	'UAH': ("Ukraine, Hryvnia","#"),
	'UGX': ("Uganda, Shillings","#"),
	'USD': ("United States of America, Dollars","$"),
	'UYU': ("Uruguay, Pesos","#"),
	'UZS': ("Uzbekistan, Sums","#"),
	'VEF': ("Venezuela, Bolivares Fuertes","#"),
	'VND': ("Viet Nam, Dong","#"),
	'VUV': ("Vanuatu, Vatu","#"),
	'WST': ("Samoa, Tala","#"),
	'XAF': ("Communaut\xc3\xa9 Financi\xc3\xa8re Africaine BEAC, Francs","#"),
	'XAG': ("Silver, Ounces","#"),
	'XAU': ("Gold, Ounces","#"),
	'XCD': ("East Caribbean Dollars","#"),
	'XDR': ("International Monetary Fund (IMF) Special Drawing Rights","#"),
	'XOF': ("Communaut\xc3\xa9 Financi\xc3\xa8re Africaine BCEAO, Francs","#"),
	'XPD': ("Palladium Ounces","#"),
	'XPF': ("Comptoirs Fran\xc3\xa7ais du Pacifique Francs","#"),
	'XPT': ("Platinum, Ounces","#"),
	'YER': ("Yemen, Rials","#"),
	'YUM': ("Yugoslavia, New Dinars","#"),
	'ZAR': ("South Africa, Rand","#"),
	'ZMK': ("Zambia, Kwacha","#"),
	'ZWD': ("Zimbabwe, Zimbabwe Dollars","#"),
    }

_isocc2c = {}
for k,v in list(_c2isocc.items()):
	v = v.upper()
	_isocc2c[v] = k
	del _c2isocc[k]
	_c2isocc[k.upper()] = v
del k, v

def iso2country(cc):
	'''map an iso code to a country'''
	return _isocc2c[cc.upper()]

def country2iso(c):
	'''map a country to an iso code'''
	return _c2isocc[c.upper()]

def ccc2sym(ccc):
	return _cccmap[ccc.upper()][1]



#Copyright ReportLab Europe Ltd. 2000-2012
#see license.txt for license details
#history www.reportlab.co.uk/rl-cgi/viewcvs.cgi/rlextra/thirdparty/__init__.py
__version__=''' $Id$ '''
# The thirdparty package exists to allow us to have other packages in a standard location
# eg we can do
#from rlextra.thirdparty.rlextra.thirdparty import xlrd



__code__='Gat=(s)6X[\'RuIZ=keq<FCd/YR+W<e83#u@\'g`0\\Z4nXrTj##2AK]1h]=)#dg[j7/WRUmr8c6G4`F$W3B7!4<`2\'j348\\I&kKC4#754=WMlr]9;ZBS/4"pa2"U-$aCZ\'HkJ<9g\\i"Yrune]@O+T7aZ%cA=7$h+5U4Q$;!,2bo*97ZI3?3\\%D\\-2Rji%j4j^_OWq!3ZEP#l#"iFhj%=568CtGSBRF^g"`tj[GBD*rU?drZH(`7WT\\VX?\\fhkMW]RioT&OI."dk@Wqi;A#U^$E#T#KccLnR\'S_aK/FqGhlY!YOLVa?_:KfhjE)_,qD!JE96@\'WYSLI=?_VqIS]"3j\\F1DFD#e9T.$/1!&1`:rLWaA%NlC"N7X;s975lS"E._)d^!^peH>S?bT7>eoe\'l6$Ygu%N_g=U:!UU`Wu:[1dB_XPK5QmcZ]<AhtagP(c53A9AkQPLRQ-eYn6;Suh\\C<n_Mge$7:b6E$K6BT?G0#Fe6G\\c14C.B"9Vjclf*@IjsG-V7ar7In03J4;V>N/)V/=(s#)gZed04j`Y&>\\8e#(]pa&*"\'m_N1&Mj%C%4e[d=tQHBQIc%<!l,`B`_&=ct.I?BJh306t&rbsPFRl%]PA=aOi%sQe\\s67=YPoI;+*6D/OFRt(u8(u42BD1_.SU]P]*$$&FKq1oS*A>RTSm*E5IH/`-3t)c9l8VGmXDm8HNse:5ZMaJK*7Ifc;l9DG/U"Ip0Y+??]JHU(gmP6fmG9[I&H62;:_nqrl7?ua8[S=g_PV6hT/BS&]L>Znh_Or.RT,&A]BQ3d>:$mOFDbuRBKoJE/(6HSjt_qj=4^L;\\%_S\'2sV!gX-T?GjYP%$Vcjibag+6[^N[p=F;\'^\\I7h`Cr2PpREi[q-ZKmu+)P_]u*Q72K?1Ds_modC$/_\'7YXt\'8FmY4l4=goOADAX`6.tHOD7pKuR%]XI9RDg2fa:tmCG+Z`V@LF.*SXI7G<o%ZgCRTXmc/XM^^$uh:/On@JL\\jYqHK-1XD+!:[r_+E7:*[c@VUP79:!E<so#u*@SGT7ijM2tsADrQV676Y3hk/=*g[fQ7$\\C\'(86DjVdo`U")/D65G2^6V__[j%$9Zjd*oX3]LU\')Aa=QU,IKAGCCI\'M-f)PYTDleaqj\'N!!U7D~>'



import difflib
import pprint
import re

from copy import deepcopy

import unittest2

from unittest2.test.support import (
    OldTestResult, EqualityMixin, HashingMixin, LoggingResult
)


class MyException(Exception):
    pass


class Test(object):
    "Keep these TestCase classes out of the main namespace"

    class Foo(unittest2.TestCase):
        def runTest(self): pass
        def test1(self): pass

    class Bar(Foo):
        def test2(self): pass

    class LoggingTestCase(unittest2.TestCase):
        """A test case which logs its calls."""

        def __init__(self, events):
            super(Test.LoggingTestCase, self).__init__('test')
            self.events = events

        def setUp(self):
            self.events.append('setUp')

        def test(self):
            self.events.append('test')

        def tearDown(self):
            self.events.append('tearDown')



class TestCleanUp(unittest2.TestCase):

    def testCleanUp(self):
        class TestableTest(unittest2.TestCase):
            def testNothing(self):
                pass

        test = TestableTest('testNothing')
        self.assertEqual(test._cleanups, [])

        cleanups = []

        def cleanup1(*args, **kwargs):
            cleanups.append((1, args, kwargs))

        def cleanup2(*args, **kwargs):
            cleanups.append((2, args, kwargs))

        test.addCleanup(cleanup1, 1, 2, 3, four='hello', five='goodbye')
        test.addCleanup(cleanup2)

        self.assertEqual(test._cleanups,
                         [(cleanup1, (1, 2, 3), dict(four='hello', five='goodbye')),
                          (cleanup2, (), {})])

        result = test.doCleanups()
        self.assertTrue(result)

        self.assertEqual(cleanups, [(2, (), {}), (1, (1, 2, 3), dict(four='hello', five='goodbye'))])

    def testCleanUpWithErrors(self):
        class TestableTest(unittest2.TestCase):
            def testNothing(self):
                pass

        class MockResult(object):
            errors = []
            def addError(self, test, exc_info):
                self.errors.append((test, exc_info))

        result = MockResult()
        test = TestableTest('testNothing')
        test._resultForDoCleanups = result

        exc1 = Exception('foo')
        exc2 = Exception('bar')
        def cleanup1():
            raise exc1

        def cleanup2():
            raise exc2

        test.addCleanup(cleanup1)
        test.addCleanup(cleanup2)

        self.assertFalse(test.doCleanups())

        (test1, (Type1, instance1, _)), (test2, (Type2, instance2, _)) = reversed(MockResult.errors)
        self.assertEqual((test1, Type1, instance1), (test, Exception, exc1))
        self.assertEqual((test2, Type2, instance2), (test, Exception, exc2))

    def testCleanupInRun(self):
        blowUp = False
        ordering = []

        class TestableTest(unittest2.TestCase):
            def setUp(self):
                ordering.append('setUp')
                if blowUp:
                    raise Exception('foo')

            def testNothing(self):
                ordering.append('test')

            def tearDown(self):
                ordering.append('tearDown')

        test = TestableTest('testNothing')

        def cleanup1():
            ordering.append('cleanup1')
        def cleanup2():
            ordering.append('cleanup2')
        test.addCleanup(cleanup1)
        test.addCleanup(cleanup2)

        def success(some_test):
            self.assertEqual(some_test, test)
            ordering.append('success')

        result = unittest2.TestResult()
        result.addSuccess = success

        test.run(result)
        self.assertEqual(ordering, ['setUp', 'test', 'tearDown',
                                    'cleanup2', 'cleanup1', 'success'])

        blowUp = True
        ordering = []
        test = TestableTest('testNothing')
        test.addCleanup(cleanup1)
        test.run(result)
        self.assertEqual(ordering, ['setUp', 'cleanup1'])

    def testTestCaseDebugExecutesCleanups(self):
        ordering = []

        class TestableTest(unittest2.TestCase):
            def setUp(self):
                ordering.append('setUp')
                self.addCleanup(cleanup1)

            def testNothing(self):
                ordering.append('test')

            def tearDown(self):
                ordering.append('tearDown')

        test = TestableTest('testNothing')

        def cleanup1():
            ordering.append('cleanup1')
            test.addCleanup(cleanup2)
        def cleanup2():
            ordering.append('cleanup2')

        test.debug()
        self.assertEqual(ordering, ['setUp', 'test', 'tearDown', 'cleanup1', 'cleanup2'])


class Test_TestCase(unittest2.TestCase, EqualityMixin, HashingMixin):

    ### Set up attributes used by inherited tests
    ################################################################

    # Used by HashingMixin.test_hash and EqualityMixin.test_eq
    eq_pairs = [(Test.Foo('test1'), Test.Foo('test1'))]

    # Used by EqualityMixin.test_ne
    ne_pairs = [(Test.Foo('test1'), Test.Foo('runTest')),
                (Test.Foo('test1'), Test.Bar('test1')),
                (Test.Foo('test1'), Test.Bar('test2'))]

    ################################################################
    ### /Set up attributes used by inherited tests


    # "class TestCase([methodName])"
    # ...
    # "Each instance of TestCase will run a single test method: the
    # method named methodName."
    # ...
    # "methodName defaults to "runTest"."
    #
    # Make sure it really is optional, and that it defaults to the proper
    # thing.
    def test_init__no_test_name(self):
        class Test(unittest2.TestCase):
            def runTest(self): raise MyException()
            def test(self): pass

        self.assertEqual(Test().id()[-13:], '.Test.runTest')

    # "class TestCase([methodName])"
    # ...
    # "Each instance of TestCase will run a single test method: the
    # method named methodName."
    def test_init__test_name__valid(self):
        class Test(unittest2.TestCase):
            def runTest(self): raise MyException()
            def test(self): pass

        self.assertEqual(Test('test').id()[-10:], '.Test.test')

    # "class unittest2.TestCase([methodName])"
    # ...
    # "Each instance of TestCase will run a single test method: the
    # method named methodName."
    def test_init__test_name__invalid(self):
        class Test(unittest2.TestCase):
            def runTest(self): raise MyException()
            def test(self): pass

        try:
            Test('testfoo')
        except ValueError:
            pass
        else:
            self.fail("Failed to raise ValueError")

    # "Return the number of tests represented by the this test object. For
    # TestCase instances, this will always be 1"
    def test_countTestCases(self):
        class Foo(unittest2.TestCase):
            def test(self): pass

        self.assertEqual(Foo('test').countTestCases(), 1)

    # "Return the default type of test result object to be used to run this
    # test. For TestCase instances, this will always be
    # unittest2.TestResult;  subclasses of TestCase should
    # override this as necessary."
    def test_defaultTestResult(self):
        class Foo(unittest2.TestCase):
            def runTest(self):
                pass

        result = Foo().defaultTestResult()
        self.assertEqual(type(result), unittest2.TestResult)

    # "When a setUp() method is defined, the test runner will run that method
    # prior to each test. Likewise, if a tearDown() method is defined, the
    # test runner will invoke that method after each test. In the example,
    # setUp() was used to create a fresh sequence for each test."
    #
    # Make sure the proper call order is maintained, even if setUp() raises
    # an exception.
    def test_run_call_order__error_in_setUp(self):
        events = []
        result = LoggingResult(events)

        class Foo(Test.LoggingTestCase):
            def setUp(self):
                super(Foo, self).setUp()
                raise RuntimeError('raised by Foo.setUp')

        Foo(events).run(result)
        expected = ['startTest', 'setUp', 'addError', 'stopTest']
        self.assertEqual(events, expected)

    # "With a temporary result stopTestRun is called when setUp errors.
    def test_run_call_order__error_in_setUp_default_result(self):
        events = []

        class Foo(Test.LoggingTestCase):
            def defaultTestResult(self):
                return LoggingResult(self.events)

            def setUp(self):
                super(Foo, self).setUp()
                raise RuntimeError('raised by Foo.setUp')

        Foo(events).run()
        expected = ['startTestRun', 'startTest', 'setUp', 'addError',
                    'stopTest', 'stopTestRun']
        self.assertEqual(events, expected)

    # "When a setUp() method is defined, the test runner will run that method
    # prior to each test. Likewise, if a tearDown() method is defined, the
    # test runner will invoke that method after each test. In the example,
    # setUp() was used to create a fresh sequence for each test."
    #
    # Make sure the proper call order is maintained, even if the test raises
    # an error (as opposed to a failure).
    def test_run_call_order__error_in_test(self):
        events = []
        result = LoggingResult(events)

        class Foo(Test.LoggingTestCase):
            def test(self):
                super(Foo, self).test()
                raise RuntimeError('raised by Foo.test')

        expected = ['startTest', 'setUp', 'test', 'addError', 'tearDown',
                    'stopTest']
        Foo(events).run(result)
        self.assertEqual(events, expected)

    # "With a default result, an error in the test still results in stopTestRun
    # being called."
    def test_run_call_order__error_in_test_default_result(self):
        events = []

        class Foo(Test.LoggingTestCase):
            def defaultTestResult(self):
                return LoggingResult(self.events)

            def test(self):
                super(Foo, self).test()
                raise RuntimeError('raised by Foo.test')

        expected = ['startTestRun', 'startTest', 'setUp', 'test', 'addError',
                    'tearDown', 'stopTest', 'stopTestRun']
        Foo(events).run()
        self.assertEqual(events, expected)

    # "When a setUp() method is defined, the test runner will run that method
    # prior to each test. Likewise, if a tearDown() method is defined, the
    # test runner will invoke that method after each test. In the example,
    # setUp() was used to create a fresh sequence for each test."
    #
    # Make sure the proper call order is maintained, even if the test signals
    # a failure (as opposed to an error).
    def test_run_call_order__failure_in_test(self):
        events = []
        result = LoggingResult(events)

        class Foo(Test.LoggingTestCase):
            def test(self):
                super(Foo, self).test()
                self.fail('raised by Foo.test')

        expected = ['startTest', 'setUp', 'test', 'addFailure', 'tearDown',
                    'stopTest']
        Foo(events).run(result)
        self.assertEqual(events, expected)

    # "When a test fails with a default result stopTestRun is still called."
    def test_run_call_order__failure_in_test_default_result(self):

        class Foo(Test.LoggingTestCase):
            def defaultTestResult(self):
                return LoggingResult(self.events)
            def test(self):
                super(Foo, self).test()
                self.fail('raised by Foo.test')

        expected = ['startTestRun', 'startTest', 'setUp', 'test', 'addFailure',
                    'tearDown', 'stopTest', 'stopTestRun']
        events = []
        Foo(events).run()
        self.assertEqual(events, expected)

    # "When a setUp() method is defined, the test runner will run that method
    # prior to each test. Likewise, if a tearDown() method is defined, the
    # test runner will invoke that method after each test. In the example,
    # setUp() was used to create a fresh sequence for each test."
    #
    # Make sure the proper call order is maintained, even if tearDown() raises
    # an exception.
    def test_run_call_order__error_in_tearDown(self):
        events = []
        result = LoggingResult(events)

        class Foo(Test.LoggingTestCase):
            def tearDown(self):
                super(Foo, self).tearDown()
                raise RuntimeError('raised by Foo.tearDown')

        Foo(events).run(result)
        expected = ['startTest', 'setUp', 'test', 'tearDown', 'addError',
                    'stopTest']
        self.assertEqual(events, expected)

    # "When tearDown errors with a default result stopTestRun is still called."
    def test_run_call_order__error_in_tearDown_default_result(self):

        class Foo(Test.LoggingTestCase):
            def defaultTestResult(self):
                return LoggingResult(self.events)
            def tearDown(self):
                super(Foo, self).tearDown()
                raise RuntimeError('raised by Foo.tearDown')

        events = []
        Foo(events).run()
        expected = ['startTestRun', 'startTest', 'setUp', 'test', 'tearDown',
                    'addError', 'stopTest', 'stopTestRun']
        self.assertEqual(events, expected)

    # "TestCase.run() still works when the defaultTestResult is a TestResult
    # that does not support startTestRun and stopTestRun.
    def test_run_call_order_default_result(self):

        class Foo(unittest2.TestCase):
            def defaultTestResult(self):
                return OldTestResult()
            def test(self):
                pass

        Foo('test').run()

    # "This class attribute gives the exception raised by the test() method.
    # If a test framework needs to use a specialized exception, possibly to
    # carry additional information, it must subclass this exception in
    # order to ``play fair'' with the framework.  The initial value of this
    # attribute is AssertionError"
    def test_failureException__default(self):
        class Foo(unittest2.TestCase):
            def test(self):
                pass

        self.assertTrue(Foo('test').failureException is AssertionError)

    # "This class attribute gives the exception raised by the test() method.
    # If a test framework needs to use a specialized exception, possibly to
    # carry additional information, it must subclass this exception in
    # order to ``play fair'' with the framework."
    #
    # Make sure TestCase.run() respects the designated failureException
    def test_failureException__subclassing__explicit_raise(self):
        events = []
        result = LoggingResult(events)

        class Foo(unittest2.TestCase):
            def test(self):
                raise RuntimeError()

            failureException = RuntimeError

        self.assertTrue(Foo('test').failureException is RuntimeError)


        Foo('test').run(result)
        expected = ['startTest', 'addFailure', 'stopTest']
        self.assertEqual(events, expected)

    # "This class attribute gives the exception raised by the test() method.
    # If a test framework needs to use a specialized exception, possibly to
    # carry additional information, it must subclass this exception in
    # order to ``play fair'' with the framework."
    #
    # Make sure TestCase.run() respects the designated failureException
    def test_failureException__subclassing__implicit_raise(self):
        events = []
        result = LoggingResult(events)

        class Foo(unittest2.TestCase):
            def test(self):
                self.fail("foo")

            failureException = RuntimeError

        self.assertTrue(Foo('test').failureException is RuntimeError)


        Foo('test').run(result)
        expected = ['startTest', 'addFailure', 'stopTest']
        self.assertEqual(events, expected)

    # "The default implementation does nothing."
    def test_setUp(self):
        class Foo(unittest2.TestCase):
            def runTest(self):
                pass

        # ... and nothing should happen
        Foo().setUp()

    # "The default implementation does nothing."
    def test_tearDown(self):
        class Foo(unittest2.TestCase):
            def runTest(self):
                pass

        # ... and nothing should happen
        Foo().tearDown()

    # "Return a string identifying the specific test case."
    #
    # Because of the vague nature of the docs, I'm not going to lock this
    # test down too much. Really all that can be asserted is that the id()
    # will be a string (either 8-byte or unicode -- again, because the docs
    # just say "string")
    def test_id(self):
        class Foo(unittest2.TestCase):
            def runTest(self):
                pass

        self.assertIsInstance(Foo().id(), str)

    # "If result is omitted or None, a temporary result object is created
    # and used, but is not made available to the caller. As TestCase owns the
    # temporary result startTestRun and stopTestRun are called.

    def test_run__uses_defaultTestResult(self):
        events = []

        class Foo(unittest2.TestCase):
            def test(self):
                events.append('test')

            def defaultTestResult(self):
                return LoggingResult(events)

        # Make run() find a result object on its own
        Foo('test').run()

        expected = ['startTestRun', 'startTest', 'test', 'addSuccess',
            'stopTest', 'stopTestRun']
        self.assertEqual(events, expected)

    def testShortDescriptionWithoutDocstring(self):
        self.assertIsNone(self.shortDescription())

    def testShortDescriptionWithOneLineDocstring(self):
        """Tests shortDescription() for a method with a docstring."""
        self.assertEqual(
                self.shortDescription(),
                'Tests shortDescription() for a method with a docstring.')

    def testShortDescriptionWithMultiLineDocstring(self):
        """Tests shortDescription() for a method with a longer docstring.

        This method ensures that only the first line of a docstring is
        returned used in the short description, no matter how long the
        whole thing is.
        """
        self.assertEqual(
                self.shortDescription(),
                 'Tests shortDescription() for a method with a longer '
                 'docstring.')

    def testAddTypeEqualityFunc(self):
        class SadSnake(object):
            """Dummy class for test_addTypeEqualityFunc."""
        s1, s2 = SadSnake(), SadSnake()
        self.assertNotEqual(s1, s2)
        def AllSnakesCreatedEqual(a, b, msg=None):
            return type(a) is type(b) is SadSnake
        self.addTypeEqualityFunc(SadSnake, AllSnakesCreatedEqual)
        self.assertEqual(s1, s2)
        # No this doesn't clean up and remove the SadSnake equality func
        # from this TestCase instance but since its a local nothing else
        # will ever notice that.

    def testAssertIs(self):
        thing = object()
        self.assertIs(thing, thing)
        self.assertRaises(self.failureException, self.assertIs, thing, object())

    def testAssertIsNot(self):
        thing = object()
        self.assertIsNot(thing, object())
        self.assertRaises(self.failureException, self.assertIsNot, thing, thing)

    def testAssertIsInstance(self):
        thing = []
        self.assertIsInstance(thing, list)
        self.assertRaises(self.failureException, self.assertIsInstance,
                          thing, dict)

    def testAssertNotIsInstance(self):
        thing = []
        self.assertNotIsInstance(thing, dict)
        self.assertRaises(self.failureException, self.assertNotIsInstance,
                          thing, list)

    def testAssertIn(self):
        animals = {'monkey': 'banana', 'cow': 'grass', 'seal': 'fish'}

        self.assertIn('a', 'abc')
        self.assertIn(2, [1, 2, 3])
        self.assertIn('monkey', animals)

        self.assertNotIn('d', 'abc')
        self.assertNotIn(0, [1, 2, 3])
        self.assertNotIn('otter', animals)

        self.assertRaises(self.failureException, self.assertIn, 'x', 'abc')
        self.assertRaises(self.failureException, self.assertIn, 4, [1, 2, 3])
        self.assertRaises(self.failureException, self.assertIn, 'elephant',
                          animals)

        self.assertRaises(self.failureException, self.assertNotIn, 'c', 'abc')
        self.assertRaises(self.failureException, self.assertNotIn, 1, [1, 2, 3])
        self.assertRaises(self.failureException, self.assertNotIn, 'cow',
                          animals)

    def testAssertDictContainsSubset(self):
        self.assertDictContainsSubset({}, {})
        self.assertDictContainsSubset({}, {'a': 1})
        self.assertDictContainsSubset({'a': 1}, {'a': 1})
        self.assertDictContainsSubset({'a': 1}, {'a': 1, 'b': 2})
        self.assertDictContainsSubset({'a': 1, 'b': 2}, {'a': 1, 'b': 2})

        self.assertRaises(unittest2.TestCase.failureException,
                          self.assertDictContainsSubset, {'a': 2}, {'a': 1},
                          '.*Mismatched values:.*')

        self.assertRaises(unittest2.TestCase.failureException,
                          self.assertDictContainsSubset, {'c': 1}, {'a': 1},
                          '.*Missing:.*')

        self.assertRaises(unittest2.TestCase.failureException,
                          self.assertDictContainsSubset, {'a': 1, 'c': 1},
                          {'a': 1}, '.*Missing:.*')

        self.assertRaises(unittest2.TestCase.failureException,
                          self.assertDictContainsSubset, {'a': 1, 'c': 1},
                          {'a': 1}, '.*Missing:.*Mismatched values:.*')
        
        self.assertRaises(self.failureException,
                          self.assertDictContainsSubset, {1: "one"}, {})

    def testAssertEqual(self):
        equal_pairs = [
                ((), ()),
                ({}, {}),
                ([], []),
                (set(), set()),
                (frozenset(), frozenset())]
        for a, b in equal_pairs:
            # This mess of try excepts is to test the assertEqual behavior
            # itself.
            try:
                self.assertEqual(a, b)
            except self.failureException:
                self.fail('assertEqual(%r, %r) failed' % (a, b))
            try:
                self.assertEqual(a, b, msg='foo')
            except self.failureException:
                self.fail('assertEqual(%r, %r) with msg= failed' % (a, b))
            try:
                self.assertEqual(a, b, 'foo')
            except self.failureException:
                self.fail('assertEqual(%r, %r) with third parameter failed' %
                          (a, b))

        unequal_pairs = [
               ((), []),
               ({}, set()),
               (set([4,1]), frozenset([4,2])),
               (frozenset([4,5]), set([2,3])),
               (set([3,4]), set([5,4]))]
        for a, b in unequal_pairs:
            self.assertRaises(self.failureException, self.assertEqual, a, b)
            self.assertRaises(self.failureException, self.assertEqual, a, b,
                              'foo')
            self.assertRaises(self.failureException, self.assertEqual, a, b,
                              msg='foo')

    def testEquality(self):
        self.assertListEqual([], [])
        self.assertTupleEqual((), ())
        self.assertSequenceEqual([], ())

        a = [0, 'a', []]
        b = []
        self.assertRaises(unittest2.TestCase.failureException,
                          self.assertListEqual, a, b)
        self.assertRaises(unittest2.TestCase.failureException,
                          self.assertListEqual, tuple(a), tuple(b))
        self.assertRaises(unittest2.TestCase.failureException,
                          self.assertSequenceEqual, a, tuple(b))

        b.extend(a)
        self.assertListEqual(a, b)
        self.assertTupleEqual(tuple(a), tuple(b))
        self.assertSequenceEqual(a, tuple(b))
        self.assertSequenceEqual(tuple(a), b)

        self.assertRaises(self.failureException, self.assertListEqual,
                          a, tuple(b))
        self.assertRaises(self.failureException, self.assertTupleEqual,
                          tuple(a), b)
        self.assertRaises(self.failureException, self.assertListEqual, None, b)
        self.assertRaises(self.failureException, self.assertTupleEqual, None,
                          tuple(b))
        self.assertRaises(self.failureException, self.assertSequenceEqual,
                          None, tuple(b))
        self.assertRaises(self.failureException, self.assertListEqual, 1, 1)
        self.assertRaises(self.failureException, self.assertTupleEqual, 1, 1)
        self.assertRaises(self.failureException, self.assertSequenceEqual,
                          1, 1)

        self.assertDictEqual({}, {})

        c = { 'x': 1 }
        d = {}
        self.assertRaises(unittest2.TestCase.failureException,
                          self.assertDictEqual, c, d)

        d.update(c)
        self.assertDictEqual(c, d)

        d['x'] = 0
        self.assertRaises(unittest2.TestCase.failureException,
                          self.assertDictEqual, c, d, 'These are unequal')

        self.assertRaises(self.failureException, self.assertDictEqual, None, d)
        self.assertRaises(self.failureException, self.assertDictEqual, [], d)
        self.assertRaises(self.failureException, self.assertDictEqual, 1, 1)

    def testAssertItemsEqual(self):
        self.assertItemsEqual([1, 2, 3], [3, 2, 1])
        self.assertItemsEqual(['foo', 'bar', 'baz'], ['bar', 'baz', 'foo'])
        self.assertRaises(self.failureException, self.assertItemsEqual,
                          [10], [10, 11])
        self.assertRaises(self.failureException, self.assertItemsEqual,
                          [10, 11], [10])
        self.assertRaises(self.failureException, self.assertItemsEqual,
                          [10, 11, 10], [10, 11])

        # Test that sequences of unhashable objects can be tested for sameness:
        self.assertItemsEqual([[1, 2], [3, 4]], [[3, 4], [1, 2]])

        self.assertItemsEqual([{'a': 1}, {'b': 2}], [{'b': 2}, {'a': 1}])
        self.assertRaises(self.failureException, self.assertItemsEqual,
                          [[1]], [[2]])
        
        # Test unsortable objects
        self.assertItemsEqual([2j, None], [None, 2j])
        self.assertRaises(self.failureException, self.assertItemsEqual,
                          [2j, None], [None, 3j])

    def testAssertSetEqual(self):
        set1 = set()
        set2 = set()
        self.assertSetEqual(set1, set2)

        self.assertRaises(self.failureException, self.assertSetEqual, None, set2)
        self.assertRaises(self.failureException, self.assertSetEqual, [], set2)
        self.assertRaises(self.failureException, self.assertSetEqual, set1, None)
        self.assertRaises(self.failureException, self.assertSetEqual, set1, [])

        set1 = set(['a'])
        set2 = set()
        self.assertRaises(self.failureException, self.assertSetEqual, set1, set2)

        set1 = set(['a'])
        set2 = set(['a'])
        self.assertSetEqual(set1, set2)

        set1 = set(['a'])
        set2 = set(['a', 'b'])
        self.assertRaises(self.failureException, self.assertSetEqual, set1, set2)

        set1 = set(['a'])
        set2 = frozenset(['a', 'b'])
        self.assertRaises(self.failureException, self.assertSetEqual, set1, set2)

        set1 = set(['a', 'b'])
        set2 = frozenset(['a', 'b'])
        self.assertSetEqual(set1, set2)

        set1 = set()
        set2 = "foo"
        self.assertRaises(self.failureException, self.assertSetEqual, set1, set2)
        self.assertRaises(self.failureException, self.assertSetEqual, set2, set1)

        # make sure any string formatting is tuple-safe
        set1 = set([(0, 1), (2, 3)])
        set2 = set([(4, 5)])
        self.assertRaises(self.failureException, self.assertSetEqual, set1, set2)

    def testInequality(self):
        # Try ints
        self.assertGreater(2, 1)
        self.assertGreaterEqual(2, 1)
        self.assertGreaterEqual(1, 1)
        self.assertLess(1, 2)
        self.assertLessEqual(1, 2)
        self.assertLessEqual(1, 1)
        self.assertRaises(self.failureException, self.assertGreater, 1, 2)
        self.assertRaises(self.failureException, self.assertGreater, 1, 1)
        self.assertRaises(self.failureException, self.assertGreaterEqual, 1, 2)
        self.assertRaises(self.failureException, self.assertLess, 2, 1)
        self.assertRaises(self.failureException, self.assertLess, 1, 1)
        self.assertRaises(self.failureException, self.assertLessEqual, 2, 1)

        # Try Floats
        self.assertGreater(1.1, 1.0)
        self.assertGreaterEqual(1.1, 1.0)
        self.assertGreaterEqual(1.0, 1.0)
        self.assertLess(1.0, 1.1)
        self.assertLessEqual(1.0, 1.1)
        self.assertLessEqual(1.0, 1.0)
        self.assertRaises(self.failureException, self.assertGreater, 1.0, 1.1)
        self.assertRaises(self.failureException, self.assertGreater, 1.0, 1.0)
        self.assertRaises(self.failureException, self.assertGreaterEqual, 1.0, 1.1)
        self.assertRaises(self.failureException, self.assertLess, 1.1, 1.0)
        self.assertRaises(self.failureException, self.assertLess, 1.0, 1.0)
        self.assertRaises(self.failureException, self.assertLessEqual, 1.1, 1.0)

        # Try Strings
        self.assertGreater('bug', 'ant')
        self.assertGreaterEqual('bug', 'ant')
        self.assertGreaterEqual('ant', 'ant')
        self.assertLess('ant', 'bug')
        self.assertLessEqual('ant', 'bug')
        self.assertLessEqual('ant', 'ant')
        self.assertRaises(self.failureException, self.assertGreater, 'ant', 'bug')
        self.assertRaises(self.failureException, self.assertGreater, 'ant', 'ant')
        self.assertRaises(self.failureException, self.assertGreaterEqual, 'ant', 'bug')
        self.assertRaises(self.failureException, self.assertLess, 'bug', 'ant')
        self.assertRaises(self.failureException, self.assertLess, 'ant', 'ant')
        self.assertRaises(self.failureException, self.assertLessEqual, 'bug', 'ant')

        # Try Unicode
        self.assertGreater('bug', 'ant')
        self.assertGreaterEqual('bug', 'ant')
        self.assertGreaterEqual('ant', 'ant')
        self.assertLess('ant', 'bug')
        self.assertLessEqual('ant', 'bug')
        self.assertLessEqual('ant', 'ant')
        self.assertRaises(self.failureException, self.assertGreater, 'ant', 'bug')
        self.assertRaises(self.failureException, self.assertGreater, 'ant', 'ant')
        self.assertRaises(self.failureException, self.assertGreaterEqual, 'ant',
                          'bug')
        self.assertRaises(self.failureException, self.assertLess, 'bug', 'ant')
        self.assertRaises(self.failureException, self.assertLess, 'ant', 'ant')
        self.assertRaises(self.failureException, self.assertLessEqual, 'bug', 'ant')

        # Try Mixed String/Unicode
        self.assertGreater('bug', 'ant')
        self.assertGreater('bug', 'ant')
        self.assertGreaterEqual('bug', 'ant')
        self.assertGreaterEqual('bug', 'ant')
        self.assertGreaterEqual('ant', 'ant')
        self.assertGreaterEqual('ant', 'ant')
        self.assertLess('ant', 'bug')
        self.assertLess('ant', 'bug')
        self.assertLessEqual('ant', 'bug')
        self.assertLessEqual('ant', 'bug')
        self.assertLessEqual('ant', 'ant')
        self.assertLessEqual('ant', 'ant')
        self.assertRaises(self.failureException, self.assertGreater, 'ant', 'bug')
        self.assertRaises(self.failureException, self.assertGreater, 'ant', 'bug')
        self.assertRaises(self.failureException, self.assertGreater, 'ant', 'ant')
        self.assertRaises(self.failureException, self.assertGreater, 'ant', 'ant')
        self.assertRaises(self.failureException, self.assertGreaterEqual, 'ant',
                          'bug')
        self.assertRaises(self.failureException, self.assertGreaterEqual, 'ant',
                          'bug')
        self.assertRaises(self.failureException, self.assertLess, 'bug', 'ant')
        self.assertRaises(self.failureException, self.assertLess, 'bug', 'ant')
        self.assertRaises(self.failureException, self.assertLess, 'ant', 'ant')
        self.assertRaises(self.failureException, self.assertLess, 'ant', 'ant')
        self.assertRaises(self.failureException, self.assertLessEqual, 'bug', 'ant')
        self.assertRaises(self.failureException, self.assertLessEqual, 'bug', 'ant')

    def testAssertMultiLineEqual(self):
        sample_text = """\
http://www.python.org/doc/2.3/lib/module-unittest.html
test case
    A test case is the smallest unit of testing. [...]
"""
        revised_sample_text = """\
http://www.python.org/doc/2.4.1/lib/module-unittest.html
test case
    A test case is the smallest unit of testing. [...] You may provide your
    own implementation that does not subclass from TestCase, of course.
"""
        sample_text_error = """\
- http://www.python.org/doc/2.3/lib/module-unittest.html
?                             ^
+ http://www.python.org/doc/2.4.1/lib/module-unittest.html
?                             ^^^
  test case
-     A test case is the smallest unit of testing. [...]
+     A test case is the smallest unit of testing. [...] You may provide your
?                                                       +++++++++++++++++++++
+     own implementation that does not subclass from TestCase, of course.
"""
        self.maxDiff = None
        for type_changer in (lambda x: x, lambda x: x.decode('utf8')):
            try:
                self.assertMultiLineEqual(type_changer(sample_text),
                                          type_changer(revised_sample_text))
            except self.failureException as e:
                # need to remove the first line of the error message
                error = str(e).encode('utf8').split('\n', 1)[1]

                # assertMultiLineEqual is hooked up as the default for
                # unicode strings - so we can't use it for this check
                self.assertTrue(sample_text_error == error)

    def testAssertSequenceEqualMaxDiff(self):
        self.assertEqual(self.maxDiff, 80*8)
        seq1 = 'a' + 'x' * 80**2
        seq2 = 'b' + 'x' * 80**2
        diff = '\n'.join(difflib.ndiff(pprint.pformat(seq1).splitlines(),
                                       pprint.pformat(seq2).splitlines()))
        # the +1 is the leading \n added by assertSequenceEqual
        omitted = unittest2.case.DIFF_OMITTED % (len(diff) + 1,)

        self.maxDiff = len(diff)//2
        try:
            self.assertSequenceEqual(seq1, seq2)
        except self.failureException as e:
            msg = e.args[0]
        else:
            self.fail('assertSequenceEqual did not fail.')
        self.assertTrue(len(msg) < len(diff))
        self.assertIn(omitted, msg)

        self.maxDiff = len(diff) * 2
        try:
            self.assertSequenceEqual(seq1, seq2)
        except self.failureException as e:
            msg = e.args[0]
        else:
            self.fail('assertSequenceEqual did not fail.')
        self.assertTrue(len(msg) > len(diff))
        self.assertNotIn(omitted, msg)

        self.maxDiff = None
        try:
            self.assertSequenceEqual(seq1, seq2)
        except self.failureException as e:
            msg = e.args[0]
        else:
            self.fail('assertSequenceEqual did not fail.')
        self.assertTrue(len(msg) > len(diff))
        self.assertNotIn(omitted, msg)

    def testTruncateMessage(self):
        self.maxDiff = 1
        message = self._truncateMessage('foo', 'bar')
        omitted = unittest2.case.DIFF_OMITTED % len('bar')
        self.assertEqual(message, 'foo' + omitted)

        self.maxDiff = None
        message = self._truncateMessage('foo', 'bar')
        self.assertEqual(message, 'foobar')

        self.maxDiff = 4
        message = self._truncateMessage('foo', 'bar')
        self.assertEqual(message, 'foobar')

    def testAssertDictEqualTruncates(self):
        test = unittest2.TestCase('assertEqual')
        def truncate(msg, diff):
            return 'foo'
        test._truncateMessage = truncate
        try:
            test.assertDictEqual({}, {1: 0})
        except self.failureException as e:
            self.assertEqual(str(e), 'foo')
        else:
            self.fail('assertDictEqual did not fail')

    def testAssertMultiLineEqualTruncates(self):
        test = unittest2.TestCase('assertEqual')
        def truncate(msg, diff):
            return 'foo'
        test._truncateMessage = truncate
        try:
            test.assertMultiLineEqual('foo', 'bar')
        except self.failureException as e:
            self.assertEqual(str(e), 'foo')
        else:
            self.fail('assertMultiLineEqual did not fail')

    def testAssertIsNone(self):
        self.assertIsNone(None)
        self.assertRaises(self.failureException, self.assertIsNone, False)
        self.assertIsNotNone('DjZoPloGears on Rails')
        self.assertRaises(self.failureException, self.assertIsNotNone, None)

    def testAssertRegexpMatches(self):
        self.assertRegexpMatches('asdfabasdf', r'ab+')
        self.assertRaises(self.failureException, self.assertRegexpMatches,
                          'saaas', r'aaaa')

    def testAssertRaisesRegexp(self):
        class ExceptionMock(Exception):
            pass

        def Stub():
            raise ExceptionMock('We expect')

        self.assertRaisesRegexp(ExceptionMock, re.compile('expect$'), Stub)
        self.assertRaisesRegexp(ExceptionMock, 'expect$', Stub)
        self.assertRaisesRegexp(ExceptionMock, 'expect$', Stub)

    def testAssertNotRaisesRegexp(self):
        self.assertRaisesRegexp(
                self.failureException, '^Exception not raised$',
                self.assertRaisesRegexp, Exception, re.compile('x'),
                lambda: None)
        self.assertRaisesRegexp(
                self.failureException, '^Exception not raised$',
                self.assertRaisesRegexp, Exception, 'x',
                lambda: None)
        self.assertRaisesRegexp(
                self.failureException, '^Exception not raised$',
                self.assertRaisesRegexp, Exception, 'x',
                lambda: None)

    def testAssertRaisesRegexpMismatch(self):
        def Stub():
            raise Exception('Unexpected')

        self.assertRaisesRegexp(
                self.failureException,
                r'"\^Expected\$" does not match "Unexpected"',
                self.assertRaisesRegexp, Exception, '^Expected$',
                Stub)
        self.assertRaisesRegexp(
                self.failureException,
                r'"\^Expected\$" does not match "Unexpected"',
                self.assertRaisesRegexp, Exception, '^Expected$',
                Stub)
        self.assertRaisesRegexp(
                self.failureException,
                r'"\^Expected\$" does not match "Unexpected"',
                self.assertRaisesRegexp, Exception,
                re.compile('^Expected$'), Stub)


    def testSynonymAssertMethodNames(self):
        """Test undocumented method name synonyms.

        Please do not use these methods names in your own code.

        This test confirms their continued existence and functionality
        in order to avoid breaking existing code.
        """
        self.assertNotEquals(3, 5)
        self.assertEquals(3, 3)
        self.assertAlmostEquals(2.0, 2.0)
        self.assertNotAlmostEquals(3.0, 5.0)
        self.assert_(True)

    def testDeepcopy(self):
        # Issue: 5660
        class TestableTest(unittest2.TestCase):
            def testNothing(self):
                pass

        test = TestableTest('testNothing')

        # This shouldn't blow up
        deepcopy(test)


if __name__ == "__main__":
    unittest2.main()



#copyright ReportLab Europe Limited. 2000-2012
#see license.txt for license details
"""This script lets you make zip files programmatically.

Usage:
    zipper.py input_spec archive_name [prefix=whatever]
        [notexts=...] [notdirs=...] [verbose=[1|0]]

    input_spec:  glob pattern, filename or directory name
    archive_name: zip file name you want, should end in .zip
    prefix: optional directory prefix which goes in the archive
            so you can have it make a directory when unzipped.

    notdirs: comma separated list of directories to exclude.
                e.g 'CVS' will exclude all directories with
                CVS in their path
    notexts: comma separated list of file extensions to ignore.

Examples:    
    zipper.py myfile.ext myfile.zip
        -> makes a zip file containing myfile.ext
    zipper.py *.py myarchive.zip
        -> zips up all files matching the pattern.

    zipper.py mydir myarchive.zip
        -> recursively zips everything in the diretory.
            The directory name is included at the top of the
            archive.

    zipper.py *.py archive.zip prefix=spam
        -> prefixes each archive name with 'spam', so
            it creates its own subdirectory when expanded.

    zipper.py mydir archive.zip notexts=pyc,pyo
        -> zips up but excludes any files ending in pyc or pyo

    zipper.py mydir archive.zip notdirs=CVS
        -> zips up but excludes any subdirectories called CVS

    C:\\code>c:\code\\rlextra\\utils\\zipper.py reportlab rl.zip
    notexts=pdf,pyc,a85 notdirs=CVS
        -> makes a fairly clean distro on my PC!
        
"""
__version__=''' $Id$ '''
import zipfile
import os
import sys
import string


def getFilesUnder(directory, includeTop=0, extensions=None):
    "recursion utility"
    filelist = []
    def walker(arg, dirname, names, extensions=extensions):
        for name in names:
            if os.path.isfile(os.path.join(dirname,name)):
                if extensions is None:
                    arg.append(dirname + os.sep + name)
                else:
                    if os.path.splitext(name)[1] in extensions:
                        arg.append(dirname + os.sep + name)
    os.path.walk(directory, walker, filelist)
    if includeTop:
        for i in range(len(filelist)):
            filelist[i] = directory + os.sep + filelist[i]
    return filelist


def run(input, output,
        verbose=0,
        prefix=None,
        notdirs = [],
        notexts = []
        ):
    if verbose:
        print('input=%s, output=%s, prefix=%s' % (input, output, prefix))
        print('notdirs=%s' % notdirs)
        print('notexts=%s' % notexts)
    if os.path.isfile(input):
        filelist = [input]
    elif os.path.isdir(input):
        filelist = getFilesUnder(input)
    else:
        import glob
        filelist = glob.glob(input)

    if len(filelist) == 0:
        print('No files found')
        print(__doc__)
        sys.exit(1)

    z = zipfile.ZipFile(output, 'w', zipfile.ZIP_DEFLATED)
    for filename in filelist:
        # check if it is banned...
        dirname, rest = os.path.split(filename)
        dirBits = string.split(string.lower(dirname), os.sep)
        skipIt = 0
        for dirBit in dirBits:
            if dirBit in notdirs:
                skipIt = 1
        if skipIt:
            if verbose:
                print('skipping %s due to excluded directory "%s"' % (filename, dirBit)) 
            continue
        root, ext = os.path.splitext(rest)
        if string.lower(ext[1:]) in notexts:
            if verbose:
                print('skipping %s due to excluded extension "%s"' % (filename, ext)) 
            continue
            

        # Still here so we want the file. Do we need to add a directory prefix?
        if prefix:
            arcname = os.path.join(prefix, filename)
        else:
            arcname = filename

        # add to zip file            
        z.write(filename, arcname)
        if verbose:
            print('compressed %s' % arcname)
    z.close()
    print('wrote',output)
    
if __name__=='__main__':
    if len(sys.argv) < 2:
        print("Usage: zipper.py input_file_or_directory output.zip")
    else:
        args = sys.argv[1:]

        # separate keywords from normal args
        positional = []
        keywords = {
            'prefix':None,
            'notdirs':'',
            'notexts':'',
            'verbose':'0'
            }
        for arg in args:
            bits = string.split(arg, '=')
            if len(bits) == 1:
                positional.append(arg)
            elif len(bits) == 2:
                key, value = bits
                keywords[key] = value

        

        input = positional[0]
        output = positional[1]
        run(input, output,
            prefix=keywords['prefix'],
            notdirs=string.split(string.lower(keywords['notdirs']),','),
            notexts=string.split(string.lower(keywords['notexts']),','),
            verbose=(keywords['verbose']=='1')
            )
        



# -*- coding: utf-8 -*-
"""Main Controller"""

from tg import expose, response

from rptlabrmlsample.lib.base import BaseController
import io
import os

__all__ = ['RootController']

def makeDoc(q):
	from rlextra.rml2pdf import rml2pdf
        rml = mergeRML(q)  
    
        buf = io.StringIO()
        
        #create the pdf
        rml2pdf.go(rml, outputFileName=buf)
        buf.reset()
        pdfData = buf.read()
        
	return pdfData

def mergeRML(personName):
    
    #merge our template and variable, using the Genshi templating 
    #engine, then return the text.
    from genshi.template import TemplateLoader

    loader = TemplateLoader(os.getcwd() + '/rptlabrmlsample/templates/')
    rml = loader.load('hello.rml')
    return rml.generate(name=personName).render('xml')


class RootController(BaseController):
    """
    The root controller for the rptlab-rml-sample application.
    """

    #@expose('rptlabrmlsample.templates.makepdf')
    @expose()
    def getpdf(self,q):
        """Sample app for RML."""
	a=makeDoc(q)
	response.headers["Content-Type"] = "application/pdf"
	response.headers["Content-disposition"] = "attachment; filename=report.pdf"
	response.headers["Content-Length"] = len(a)

        return a #dict(page='makepdf')

    @expose('rptlabrmlsample.templates.makepdf')
    def default(self):
        """Sample app for RML."""

        return dict(page='makepdf')




#copyright ReportLab Inc. 2001-2012
#see license.txt for license details
"""<i>Invoice generation example</i>"""
import sys
import os

from rlextra.radxml import xmlutils
import preppy
from rlextra.rml2pdf import rml2pdf

def go(dictionary):
    """This provides support for the web framework.  Ignore it
    if you want to learn the basic concepts, instead use the command
    line routine below."""
    from rlextra.ers.framework import TemplateSelectingWrappedXMLProcessor
    return TemplateSelectingWrappedXMLProcessor().go(dictionary)


def runCommandLine(xmlFileName):

    #read into memory
    xmlText = open(xmlFileName, 'r').read()

    import pyRXPU
    tagTree = pyRXPU.Parser(ReturnUTF8=1).parse(xmlText)

    #use our 'TagWrapper' to get an object with friendly
    #syntax for referring to XML nodes
    wrappedTagTree = xmlutils.TagWrapper(tagTree)

    #make a dictionary to pass into preppy as its namespace.
    #you could pass in any Python objects or variables,
    #as long as the template expressions evaluate
    nameSpace = {'data':wrappedTagTree}

    template = preppy.getModule('rmltemplate.prep')
    rmlText = template.getOutput(nameSpace)

    #convert to PDF
    pdfFileName = os.path.splitext(xmlFileName)[0] + '.pdf'

    rml2pdf.go(rmlText, outputFileName=pdfFileName)
    print('saved %s' % pdfFileName)
    
        
if __name__=='__main__':
    if len(sys.argv) != 2:
        print('usage:  invoice.py myinvoice.xml > myinvoice.pdf')
    else:
        xmlFileName = sys.argv[1]
        runCommandLine(xmlFileName)        



#copyright ReportLab Inc. 2000-2012
#see license.txt for license details
from __future__ import print_function
__version__=''' $Id$ '''
'''
Attempt at making a fairly general pdf --> jpeg conversion using Ghostscript

'''
__all__=('normalize_gs_device','gsExeFind','epsBBox','pdf2jpeg','epsConvert','gsConvert','pdfConvert','pdfBBox')
import sys, os, glob, subprocess, re
from reportlab.lib.utils import open_and_read

def _cleanFiles(pat):
    try:
        list(map(os.remove, glob.glob(pat)))
    except:
        pass
    return glob.glob(pat)

def gsExeFind(gsExe=None):
    if not gsExe:
        from rlextra.utils.buildutils import find_exe
        known_places = []
        if sys.platform=='win32':
            exeName = 'gswin32c'
            known_places = (
                    list(sorted(glob.glob(r'C:\Program Files\gs\gs*\bin')))
                    +list(sorted(glob.glob(r'c:\Program Files (x86)\gs\gs*\bin')))
                    )
            if 'RL_GS_BIN' in os.environ:
                known_places.append(os.environ['RL_GS_BIN'])
        else:
            exeName = 'gs'
        gsExe = find_exe(exeName,known_places=known_places)
    if sys.platform in ('win32',):
        gsExe = gsExe.strip('"')
    return gsExe

_device2ext = dict( pswrite='ps',epswrite='eps',pdfwrite='pdf',jpeg='jpg',
                    png16m='png',pnggray='png',png256='png',png16='png',pngmono='png',
                    tiffgray='tif', tiff12nc='tif', tiff24nc='tif', tiff32nc='tif',
                    tiffsep='tif', tiffcrle='tif', tiffg3='tif', tiffg32d='tif',
                    tiffg4='tif', tifflzw='tif', tiffpack='tif',
                    bmpmono='bmp',bmpgray='bmp',bmpsep1='bmp',bmpsep8='bmp',
                    bmp16='bmp',bmp256='bmp',bmp16m='bmp',bmp32b='bmp',
                    )

def normalize_gs_device(odevice):
    K = list(_device2ext.keys())
    device = str(odevice).lower()
    if device not in K:
        d = device+'write'
        if d in K:
            device = d
        else:
            raise ValueError('Unsupported gs device "%s"' % odevice)
    return device

def _checkOutputFiles(pat,fn,out,err,verbose=0):
    F = glob.glob(pat)
    if not F:
        raise ValueError('No files produced from "%s"\ngs returned this output\n"""%s"""\nand error"""%s"""' %(fn,out,err))
    if verbose>1 and out:
        print(out, file=sys.stderr)
    if verbose and err:
        print(err, file=sys.stderr)
    return F

def _prepare(fn,outDir,ext,device,outfile=None,gsExe=None):
    if outDir is None:
        outDir = os.path.dirname(fn)
        if not outDir:
            outDir = '.'
    if outDir=='.':
        outDir = os.getcwd()
    if ext is None:
        ext=_device2ext[device]
    if outfile:
        outfile = os.path.splitext(os.path.basename(outfile))[0]
        if '%' in outfile:
            outfile = outfile.replace('%','_')+'%04d'
    else:
        outfile = os.path.splitext(os.path.basename(fn))[0].replace('%','_')+'-page%04d'
    outfile = os.path.join(outDir,outfile+'.'+ext)
    pat = outfile.replace('%04d','*')
    F=_cleanFiles(pat)
    if F:
        raise ValueError('Cannot clean output files for "%s"' % fn)
    return gsExeFind(gsExe),pat,outfile

def _run(ARGS,input=None):
    p = subprocess.Popen(ARGS,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE,shell=False)
    out, err = p.communicate()
    return out,err

def pdf2jpeg(fn,scale=0.125, outDir='.', gsExe=None, device='jpeg', ext=None):
    '''
    Create scaled jpeg versions of the pages in a pdf file.
    The filename will be
        basename-page0001.jpg for the first page
        basename-page0002.jpg for the second page etc

    fn      filename of the pdf(or eps) to be converted
    scale   the scalefactor default 1/8th, ignored for
            epswrite/pdfwrite device
    outDir  the folder to place outputs
            None = parallel
            default = '.'
    gsExe   The path to the ghostscript exe file
            if None we attempt to find it ourselves
    device  the output device name defaults to jpeg
            another suitable device is pdfwrite
    ext     the output ext the default is None
            if None then we use .jpg for jpeg and pdf
            for pdfwrite
    '''
    outfile=None
    return pdfConvert(fn, None, scale, outDir, device, ext,True,outfile,'art',0,gsExe)

def epsBBox(fn,bbpat=re.compile(r'^\s*%%boundingbox:\s*(.*)\s*$',re.I|re.M)):
    '''
    read an eps file and attempt to obtain its bounding box

    >>> cleanup()
    >>> x=pdfConvert('zzzdingo.pdf',scale=0.5,device='eps',outfile='zzzdingo.eps')
    >>> assert x and os.path.isfile(os.path.basename(x[0]))
    >>> print epsBBox('zzzdingo.eps')
    [0, 0, 100, 50]
    '''
    text = open_and_read(fn)
    m=bbpat.search(text)
    if not m:
        raise ValueError('Cannot find bounding box in "%s"' % fn)
    bb = m.group(1).strip().split()
    if len(bb)!=4: 
        raise ValueError('Not enough bounding box entries "%s" found in "%s"' % (m.group(0).strip(),fn))
    try:
        return list(map(float,bb))
    except:
        raise ValueError('Bad bounding box entries "%s" found in "%s"' % (m.group(0).strip(),fn))

def _ffmt(f):
    return ('%.6f' % f).rstrip('0').rstrip('.')

def gsConvert(fn,bb,size,scale,outDir,device,ext,preserveAspectRatio,outfile,gsExe,gsExtraArgs=[],pdfMode=0,verbose=0, antialiasing=4, quality=85, useCropBox=0):
    device = normalize_gs_device(device)
    owidth = bb[2]-bb[0]
    oheight = bb[3]-bb[1]
    if not size:
        if scale:
            if isinstance(scale,float):
                xscale = yscale = scale
            else:
                xscale, yscale = scale
        else:
            xscale = yscale = 1
        width = owidth*xscale
        height = oheight*yscale
    else:
        width,height = size
        width = (width is None and owidth or min(width,1))
        height = (height is None and oheight or min(height,1))
        xscale = width/float(owidth)
        yscale = height/float(oheight)
    if preserveAspectRatio and xscale!=yscale:
        xscale = yscale = min(xscale,yscale)
        width = owidth * xscale
        height = oheight * yscale
    if device in ('png','jpeg'):
        width = int(width)
        height = int(height)
    gsExe,pat,outfile = _prepare(fn,outDir,ext,device,outfile,gsExe)
    ARGS = [gsExe, '-q', '-dSAFER', '-dNOPAUSE', '-dBATCH', '-dTextAlphaBits=%d'%antialiasing, '-dGraphicsAlphaBits=%d'%antialiasing, '-dJPEGQ=%d'%quality, '-sOutputFile=%s'%outfile, '-sDEVICE=%s'%device]
    if not pdfMode: ARGS += ['-dDEVICEWIDTHPOINTS='+_ffmt(width),'-dDEVICEHEIGHTPOINTS='+_ffmt(height),'-DFIXEDMEDIA']
    if useCropBox: ARGS += ['-dUseCropBox']
    ARGS += gsExtraArgs
    if device=='pdfwrite': ARGS += ['-c','.setpdfwrite']
    if pdfMode:
        ARGS += ['-r%sx%s' % (_ffmt(72*xscale),_ffmt(72*yscale))]
    else:
        ARGS += ['-c','%s %s translate %s %s scale' % (_ffmt(-bb[0]*xscale),_ffmt(-bb[1]*yscale),_ffmt(xscale),_ffmt(yscale))]
    ARGS += ['-f',fn]
    out,err = _run(ARGS)
    return _checkOutputFiles(pat,fn,out,err,verbose=verbose)

def epsConvert(fn, size=None, scale=None, outDir='.', device='jpeg', ext=None, preserveAspectRatio=True,outfile=None,gsExe=None,gsExtraArgs=[],verbose=0):
    '''
    Convert an eps file to another format using GS
    size either None or (width,height)
    scale either None or (scalex,scaley) or scalexandy
    outDir is where the file(s) will be made
    device is the desired conversion mode (jpeg,png,pdf,ps,eps etc etc)
    ext desired extension or None to get an appropriate one based on the device
    preserveAspectRatio set to false to get skewed output that exactly fits your specified width x height
    outfile None to get an automatically named file or use the extensionless part of outfile
    gsExe   path to your gs program or None when it will be searched for
    gsExtraArgs extra arguments passed to gs

    If you don't set size then scale will be used. If unset the scale will default to 1 for both x and y.

    >>> cleanup()
    >>> x=pdfConvert('zzzdingo.pdf',scale=0.5,device='eps',outfile='zzzdingo.eps')
    >>> assert x and os.path.basename(x[0])=='zzzdingo.eps' and os.path.isfile('zzzdingo.eps')
    >>> x=epsConvert('zzzdingo.eps',scale=0.5,device='jpeg',outfile='zzzdingo.jpg')
    >>> assert x and os.path.basename(x[0])=='zzzdingo.jpg' and os.path.isfile('zzzdingo.jpg')
    >>> x=epsConvert('zzzdingo.eps',scale=0.5,device='pdf',outfile='zzzdingo1.jpg')
    >>> assert x and os.path.basename(x[0])=='zzzdingo1.pdf' and os.path.isfile('zzzdingo1.pdf')
    '''
    bb = epsBBox(fn)
    return gsConvert(fn,bb,size,scale,outDir,device,ext,preserveAspectRatio,outfile,gsExe,gsExtraArgs,verbose=verbose)

def pdfBBox(fn,boxName='art',pageNo=0):
    ''' Bounding box for a pdf

    >>> cleanup()
    >>> print pdfBBox('zzzdingo.pdf')
    [0, 0, 200, 100]
    '''
    from rlextra.pageCatcher.pdfexplorer import PdfExplorer
    boxName = boxName.capitalize()
    if boxName.endswith('box'):
        boxName = boxName[-3:]+'Box'
    elif not boxName.endswith('Box'):
        boxName += 'Box'
    meth = 'get'+boxName
    if not hasattr(PdfExplorer,meth):
        raise ValueError('No box method for "%s" in PdfExplorer' % boxName)
    p = PdfExplorer(fn)
    try:
        return getattr(p,meth)(pageNo)
    except KeyError:
        try:
            return p.getMediaBox(pageNo)
        except:
            et,ev = list(map(str,sys.exc_info()[:2]))
            raise ValueError('method %s failed for file "%s"\n%s:%s' % (meth,fn,et,ev))

def pdfConvert(fn, size=None, scale=None, outDir='.', device='jpeg', ext=None, preserveAspectRatio=True,outfile=None, boxName='art',pageNo=0,gsExe=None,gsExtraArgs=[],verbose=0, antialiasing=4, quality=85, useCropBox=0):
    '''
    Convert a pdf file to another format use GS
    size either None or (width,height)
    scale either None or (scalex,scaley) or scalexandy
    outDir is where the file(s) will be made
    device is the desired conversion mode (jpeg,png,pdf,ps,eps etc etc)
    ext desired extension or None to get an appropriate one based on the device
    preserveAspectRatio set to false to get skewed output that exactly fits your spcified width x height
    outfile None to get an automatically named file or use the extensionless part of outfile
    boxName which of the many boxes to use. if the first fails we drop back to media
    pageNo which page to get the box for
    gsExe   path to your gs program or None when it will be searched for
    gsExtraArgs extra arguments passed to gs

    If you don't set size then scale will be used. If unset the scale will default to 1 for both x and y.

    >>> cleanup()
    >>> x=pdfConvert('zzzdingo.pdf',scale=0.5,device='jpeg')
    >>> assert x and os.path.isfile(os.path.basename(x[0]))
    >>> x=pdfConvert('zzzdingo.pdf',scale=0.5,device='eps',outfile='zzzdingo.eps')
    >>> assert x and os.path.isfile(os.path.basename(x[0]))
    '''
    if isinstance(useCropBox,str): useCropBox=int(useCropBox)
    bb = pdfBBox(fn,boxName,pageNo)
    return gsConvert(fn,bb,size,scale,outDir,device,ext,preserveAspectRatio,outfile,gsExe,gsExtraArgs,pdfMode=1,verbose=verbose, antialiasing=antialiasing, quality=quality, useCropBox=useCropBox)

def test():
    from reportlab.pdfgen.canvas import Canvas
    c = Canvas('zzzdingo.pdf',(200,100))
    c.drawString(72,72,'Hello World')
    c.showPage()
    c.save()
    def cleanup(keepBase=True):
        from .buildutils import kill
        for x in glob.glob('zzzdingo*.*'):
            if keepBase and x=='zzzdingo.pdf': continue
            kill(x)
    import doctest
    doctest.testmod(extraglobs=dict(os=os,cleanup=cleanup))
    cleanup(0)

if __name__=='__main__':
    device = 'jpeg'
    boxName = 'art'
    scale = 0.125
    verbose = 0
    useCropBox = 0
    argv = sys.argv[1:]

    if '--help' in argv:
        print("""pdf2jpeg options pdffile
    options
    --scale=factor
            the scalefactor default 1/8th, ignored for
            epswrite/pdfwrite device
    --outDir=dir
            the folder to place outputs
            None = parallel
            default = '.'
    --device=jpeg|pdf
            the output device name defaults to jpeg
            another suitable device is pdfwrite
    --boxName=box
            pdfbox name default is art
    --verbose=value
            verbosity""")
        sys.exit(0)

    if not len(argv):
        test()
    else:
        for a,d in (('device','jpeg'),('boxName','art'),
                    ('scale',0.125),('verbose',0),
                    ('useCropBox',0),
                    ('outDir','.'),
                    ):
            ox = '--'+a+'='
            x = [x for x in argv if x.startswith(ox)]
            list(map(argv.remove,x))
            v = x and x[0].lstrip(ox) or d
            v = type(d)(v)
            globals()[a] = v

        for x in argv:
            func = x.lower().endswith('.eps') and epsConvert or pdfConvert
            kwds = dict(size=None, scale=scale,
                    outDir=outDir, device=device,
                    ext=None, preserveAspectRatio=True,
                    outfile=None,
                    gsExe=None,gsExtraArgs=[],
                    verbose=verbose,
                    )
            if func==pdfConvert:
                kwds.update(dict(pageNo=0,boxName=boxName,useCropBox=useCropBox))
            func(x,**kwds)
